/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <utility>
#include <stack>
#include <limits>

#include <QLabel>
#include <QHBoxLayout>
#include <QTextEdit>
#include <QCheckBox>
#include <QButtonGroup>
#include "LoadFiltersWidget.h"
#include "Service.h"
#include "TraceLoadEnterFilter.h"
#include "TraceLoadLeaveFilter.h"



LoadFiltersWidget::LoadFiltersWidget(Trace * _trace, QWidget * _parent):QWidget(_parent), trace(_trace)
{
	trace_load_enter_filter = new TraceLoadEnterFilter();
	trace_load_leave_filter = new TraceLoadLeaveFilter();
    
    
        QVBoxLayout* main_layout = new QVBoxLayout();       
        this->setLayout( main_layout );
        
        QHBoxLayout* presets_layout = new QHBoxLayout();       
        main_layout->addLayout( presets_layout );
        main_layout->setStretch(0,0);
        
        QButtonGroup * buttons_group =new QButtonGroup();
        buttons_group->setExclusive(true);
        full_load = new QPushButton("Full Load");
        full_load->setCheckable(true);
        buttons_group->addButton(full_load);
        presets_layout->addWidget(full_load);
        connect( full_load, SIGNAL(clicked()), this, SLOT(setFullLoad()) );
        
        
        preview_load = new QPushButton("Preview Load");
        preview_load->setCheckable(true);
        buttons_group->addButton(preview_load);
        presets_layout->addWidget(preview_load);
        
        connect( preview_load, SIGNAL(clicked()), this, SLOT(setPreviewLoad()) );
        
        custom_load = new QPushButton("Custom Load");
        custom_load->setCheckable(true);
        buttons_group->addButton(custom_load);
        presets_layout->addWidget(custom_load);
        connect( custom_load, SIGNAL(clicked()), this, SLOT(setCustomLoad()) );
        
        QGridLayout* settings_layout = new QGridLayout();       
        main_layout->addLayout( settings_layout );
        main_layout->setStretch(1,1);
        
        QLabel * get_start_timestamp_label = new QLabel("Start timestamp:");
        settings_layout->addWidget(get_start_timestamp_label,0,0);
            
        get_start_timestamp_input = new QLineEdit();
        settings_layout->addWidget(get_start_timestamp_input, 0,1);
        
        QLabel * get_end_timestamp_label = new QLabel("End timestamp:");
        settings_layout->addWidget(get_end_timestamp_label,1,0);
            
        get_end_timestamp_input = new QLineEdit();
        settings_layout->addWidget(get_end_timestamp_input,1,1);
                
        QLabel * call_stack_label = new QLabel("Call stack depth:");
        settings_layout->addWidget(call_stack_label,2,0);
            
        call_stack_input = new QSpinBox();
        settings_layout->addWidget(call_stack_input,2,1);
                
        QLabel * region_name_label = new QLabel("Region name (regex):");
        settings_layout->addWidget(region_name_label,3,0);
            
        region_name_input = new QLineEdit();
        settings_layout->addWidget(region_name_input,3,1);

        QLabel * region_paradigm_label = new QLabel("Region paradigms:");
        settings_layout->addWidget(region_paradigm_label,4,0);
            
        region_paradigm_input = new QLineEdit();
        settings_layout->addWidget(region_paradigm_input,4,1);
        
        QLabel * region_role_label = new QLabel("Region role:");
        settings_layout->addWidget(region_role_label,5,0);
            
        region_role_input = new QLineEdit();
        settings_layout->addWidget(region_role_input,5,1);   
        
        QLabel * resolution_label = new QLabel("Ignore resolution:");
        settings_layout->addWidget(resolution_label,6,0);
            
        resolution_check = new QCheckBox();
        settings_layout->addWidget(resolution_check,6,1);           

        QLabel * processing_label = new QLabel("Group events:");
        settings_layout->addWidget(processing_label,7,0);
            
        processing_check = new QCheckBox();
        settings_layout->addWidget(processing_check,7,1);       
        processing_check->setChecked(false);
        
        QHBoxLayout* load_layout = new QHBoxLayout();       
        main_layout->addLayout( load_layout );
        main_layout->setStretch(2,0);
        
        load_trace = new QPushButton(QIcon(":/bladeicons/load_trace_button.png"), "Load trace");
        load_layout->addWidget(load_trace);
        connect( load_trace, SIGNAL(clicked()), this, SLOT( startLoadingTrace()));
        
        setFullLoad();
        full_load->setChecked(true);
        
        
}
LoadFiltersWidget::~LoadFiltersWidget()
{
    delete trace_load_enter_filter;
    delete trace_load_leave_filter;
}
	  
 
bool
LoadFiltersWidget::do_grouping() const
{
    return processing_check->isChecked();
}
 
 
void
LoadFiltersWidget::startLoadingTrace()   
{
    trace_load_enter_filter->init( get_start_timestamp_input->text().toULong(), get_end_timestamp_input->text().toULong(), call_stack_input->value()/*, region_name_input->text(), region_paradigm_input->text(), region_role_input->text()*/ );
    
     trace_load_leave_filter->init( get_start_timestamp_input->text().toULong(), get_end_timestamp_input->text().toULong(), (resolution_check->isChecked())?-1:width());
      
    load_trace->setEnabled(false);
    emit loadTrace();
}



TraceLoadEnterFilter *
LoadFiltersWidget::get_trace_load_enter_filter()   
{
    return trace_load_enter_filter;
}

TraceLoadLeaveFilter *
LoadFiltersWidget::get_trace_load_leave_filter()   
{
    return trace_load_leave_filter;
}


 
void
LoadFiltersWidget::setFullLoad()   
{
    get_start_timestamp_input->setEnabled(false);
    get_start_timestamp_input->setText( QString("0"));
    get_end_timestamp_input->setEnabled(false);
    get_end_timestamp_input->setText( QString("%1").arg( trace->get_trace_length()));
    call_stack_input->setMinimum(-1);
    call_stack_input->setMaximum(std::numeric_limits<int>::max());
    call_stack_input->setValue(-1);
    call_stack_input->setEnabled(false);
    resolution_check->setChecked(true);
    region_name_input->setEnabled(false);
    region_paradigm_input->setEnabled(false);
    region_role_input->setEnabled(false);
}


 
void
LoadFiltersWidget::setPreviewLoad()   
{
    get_start_timestamp_input->setEnabled(false);
    get_start_timestamp_input->setText( QString("0"));
    get_end_timestamp_input->setEnabled(false);
    get_end_timestamp_input->setText( QString("%1").arg( trace->get_trace_length()));
    call_stack_input->setMinimum(-1);
    call_stack_input->setMaximum(std::numeric_limits<int>::max());
    call_stack_input->setValue(3);
    call_stack_input->setEnabled(false);
    resolution_check->setChecked(false);
    region_name_input->setEnabled(false);
    region_paradigm_input->setEnabled(false);
    region_role_input->setEnabled(false);
}


 
void
LoadFiltersWidget::setCustomLoad()   
{
    get_start_timestamp_input->setEnabled(true);
    get_start_timestamp_input->setText( QString("0"));
    get_end_timestamp_input->setEnabled(true);
    get_end_timestamp_input->setText( QString("%1").arg( trace->get_trace_length()));
    call_stack_input->setMinimum(-1);
    call_stack_input->setMaximum(std::numeric_limits<int>::max());
    call_stack_input->setValue(-1);
    call_stack_input->setEnabled(true);
    resolution_check->setChecked(true);
    region_name_input->setEnabled(true);
    region_paradigm_input->setEnabled(true);
    region_role_input->setEnabled(true);
}


