/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <utility>

#include <QLabel>
#include <QRectF>
#include <QHBoxLayout>
#include <QGraphicsItem>
#include "OverviewTraceWidget.h"
#include "Service.h"
#include "TraceEventRect.h"
#include "RoleFilter.h"
#include "RegionPropertiesFilter.h"

extern RoleFilter * role_filter;
extern RegionPropertiesFilter * rp_filter;

OverviewTraceWidget::OverviewTraceWidget(uint64_t _location, Trace * _trace, TraceEventFilter **_cf, SelectionFilter * _s, ZoomFilter * _z, ZoomFilter * _z_o, QWidget * parent):QWidget(parent), z_filters(_z),z_filter(_z_o), s_filter(_s), cf_filter(_cf) , trace(_trace), location(_location)
{
   l_filter = new LocationFilter( location );
        
    QHBoxLayout* layout = new QHBoxLayout();
    this->setLayout( layout );
    
    scene = new QGraphicsScene();
    view  = new OverviewGraphicsView(scene, z_filter, trace);
    view->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
    view->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );

    
    layout->addWidget( view );
  

    r_filter = new ResolutionFilter();
    z_filter->add_as_child( r_filter );
    
    coloring = new ColorPalette();
    updateScene();
    connect(z_filter, SIGNAL(filterChanged()),
        this, SLOT(updateScene())); 
    connect(role_filter, SIGNAL(filterChanged()),
            this, SLOT(updateSceneFull()));  
    connect(rp_filter, SIGNAL(filterChanged()),
            this, SLOT(updateSceneFull()));     
    connect(z_filters, SIGNAL(filterChanged(QPointF)),
        view, SLOT(changeZoom(QPointF)));      
    connect(view, SIGNAL(zoomChanged()),
        this, SLOT(changeZoom()));   
    connect( view, SIGNAL(zoomingSelection()),
                this, SLOT(zoomingTip()));   
    
    trace_view = trace->get_trace_view(location, z_filter, *cf_filter, s_filter, r_filter,l_filter, role_filter, rp_filter);  
    recalculateCallTreeStep();
    
}
    
     
OverviewTraceWidget::~OverviewTraceWidget()
{
    delete coloring;
    delete l_filter;
}


void
OverviewTraceWidget::changeZoom()
{
    QPointF _p = view->getZoomF();
    Point _pp = z_filter->get_window( _p ); 
    z_filters->set_zoom_window( _pp.first, _pp.second );
}

void
OverviewTraceWidget::setLocation( uint64_t _location)
{
    location = _location;
    l_filter->set_location( _location );
    updateScene( true );
}


void
OverviewTraceWidget::zoomingTip()
{
    QPointF _p = view->getZoomF();
    uint64_t _s = z_filter->get_time_in_window( _p.x());
    uint64_t _f = z_filter->get_time_in_window( _p.y());
    std::pair<double, uint16_t> _s_in_seconds = trace->convert_to_seconds( _s );
    std::pair<double, uint16_t> _f_s_in_seconds = trace->convert_to_seconds( _f - _s );
    QString _tip = (QString(" From %1%2s").arg(_s_in_seconds.first).arg( QString::fromStdString(bladeservices::convertToUnit (_s_in_seconds.second)) ) +
    QString("+%1%2s").arg(_f_s_in_seconds.first).arg( QString::fromStdString(bladeservices::convertToUnit (_f_s_in_seconds.second)), 0, 'f' )
   );
    view->showToolTip( _tip );
    
}
      
void 
OverviewTraceWidget::recalculateCallTreeStep()
{
    calltree_height_step = (int32_t)floor(std::max( 43. / ((double)(trace->get_max_level(location )+2)), 1.)) ;
    calltree_height = calltree_height_step *  (trace->get_max_level(location)+3) ;
    
    setMaximumHeight(calltree_height );     
    setMinimumHeight(calltree_height);     
    view->setMaximumHeight(calltree_height );     
    view->setMinimumHeight(calltree_height);    
    
   

    
}


  
void 
OverviewTraceWidget::resizeEvent(QResizeEvent *event)
{
    recalculateCallTreeStep();
    updateScene( );
}


void 
OverviewTraceWidget::updateSceneFull()
{
    updateScene(true);
}

void 
OverviewTraceWidget::updateScene( bool full)
{
    reduced_trace_view.clear();
    reduced_levels_trace_view.clear();
    for ( int i=0; i< trace->get_max_level(location ); ++i )
    {
        reduced_trace_view.resize(i+1);
        reduced_levels_trace_view.resize( i + 1);
        reduced_levels_trace_view[i].clear();
        reduced_trace_view[i].clear();
    }
    
//     r_filter->set_available_pixels( 100*view->width() );
    scene->clear();
    view->centerOn(QPointF(view->width()/2., view->height()/2.) );
    scene->addRect(QRectF(0  ,0 , view->width() , view->height()));
    
    if (full)
    {
        trace_view.clear();
        trace_view = trace->get_trace_view(location, z_filter, *cf_filter, s_filter, r_filter,l_filter, role_filter, rp_filter);    
    }
    
    for( size_t i=0; i<trace_view.size() ; ++i)
    {
        TraceEvent * _te = trace_view[i];
        
        std::pair<QPen, QBrush> _colors = coloring->getColoring( _te) ;

        double _normalized_start =  (_te->get_start_timestamp() > z_filter->get_zoom_start())?
                                    ((double)(_te->get_start_timestamp() - z_filter->get_zoom_start())
                                    /
                                    (double)( z_filter->get_zoom_size() )):0;
                                    
        double _normalized_end =    (_te->get_end_timestamp()  < z_filter->get_zoom_size()+ z_filter->get_zoom_start())?
                                    ((double)(_te->get_end_timestamp() -  z_filter->get_zoom_start())
                                    /
                                    (double)(z_filter->get_zoom_size() )):1.;

        std::pair<uint64_t, uint64_t> _point=std::make_pair( (uint64_t)(_normalized_start * view->width()), (uint64_t)(_normalized_end * view->width()));
        QGraphicsRectItem * _rect = NULL;
        if ( _te->is_exclusive())
        {
            QPen pen;
            pen.setColor(Qt::white);
            QBrush brush;
            brush.setColor( Qt::white );
            brush.setStyle( Qt::SolidPattern );
             _rect = new QGraphicsRectItem( _point.first,_te->get_level()*calltree_height_step, _point.second - _point.first , calltree_height_step );
            _rect->setPen( pen );
            _rect->setBrush( brush );
        }
        else
        {
            _rect = new TraceEventRect( trace, _te, QRect( _point.first ,_te->get_level()*calltree_height_step, _point.second - _point.first , calltree_height_step) );
            _rect->setPen( _colors.first );
            _rect->setBrush( _colors.second );
        }
        reduced_trace_view[_te->get_level()-1].push_back(_rect);  

    }
    createReducedTraceView( );
    

    
    
    std::vector< std::vector<  QGraphicsRectItem * > >::iterator level_iter = reduced_levels_trace_view.begin();
    int level = 0;
    for (;  level_iter!= reduced_levels_trace_view.end(); ++level_iter, ++level )
    {
        std::vector<  QGraphicsRectItem * > & _rects = *level_iter;
        std::vector<  QGraphicsRectItem * >::iterator rects_iter = _rects.begin();
        for (;  rects_iter!= _rects.end(); ++rects_iter )
        {
            QGraphicsRectItem* _rect = *rects_iter;
            scene->addItem( _rect );
        }
    }
    
    view->updateScene();
    
}


void 
OverviewTraceWidget::createReducedTraceView()
{
    std::vector< std::vector<  QGraphicsRectItem * > >::iterator level_iter = reduced_trace_view.begin();
    int level = 0;
    for (;  level_iter!= reduced_trace_view.end(); ++level_iter, ++level )
    {
        
        std::vector<  QGraphicsRectItem * > & _rects = *level_iter;
        std::vector<  QGraphicsRectItem * >::iterator rects_iter = _rects.begin();
        
        for (;  rects_iter!= _rects.end(); ++rects_iter )
        {
            QGraphicsRectItem* _rect = *rects_iter;
            if ( reduced_levels_trace_view[ level].size() == 0 )
            {
                reduced_levels_trace_view[level].push_back( _rect);
                continue;
            }
            else // comparing with already stored rects
            {
                QGraphicsRectItem* last_rect = reduced_levels_trace_view[ level].back();
                if ( _rect->rect().intersects( last_rect->rect()))
                    last_rect->setRect( _rect->rect().united( last_rect->rect()));
                else
                    reduced_levels_trace_view[ level].push_back( _rect);
            }
        }
    }
}


