/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <utility>
#include <iomanip>

#include <QLabel>
#include <QRectF>
#include <QHBoxLayout>
#include <QGraphicsItem>
#include "SubTraceWidget.h"
#include "Service.h"
#include "TraceEventRect.h"


SubTraceWidget::SubTraceWidget(uint64_t _location, QWidget * parent):QWidget(parent), location(_location)
{
        l_filter = new LocationFilter( location );
    
    
        QHBoxLayout* layout = new QHBoxLayout();
        this->setLayout( layout );
       
        scene = new QGraphicsScene();
        view  = new GraphicsView(scene);
        view->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
        view->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
        
        layout->addWidget( view );
        
	compactToggle( false );
        r_filter = new ResolutionFilter();
        z_filter->add_as_child( r_filter );
        
        coloring = new ColorPalette();
        updateScene();
        
        connect(view, SIGNAL(zoomChanged()),
                this, SLOT(changeZoom()));
        connect(z_filter, SIGNAL(filterChanged()),
                this, SLOT(updateScene()));  
        connect(role_filter, SIGNAL(filterChanged()),
                this, SLOT(updateScene()));  
        connect(rp_filter, SIGNAL(filterChanged()),
                this, SLOT(updateScene()));  
        
        connect( view, SIGNAL(zoomingSelection()),
                 this, SLOT(zoomingTip()));
        connect( view, SIGNAL(cursorMoved(QPointF)),
                 this, SLOT(updateCursor(QPointF)));     
        
        metric_trace_widget = NULL;
        if ( bladeservices::thread_id( location) == 0)
        {
            metric_trace_widget = new MetricTraceWidget();
            metric_trace_widget->hide();
        }
}
    
    
SubTraceWidget::~SubTraceWidget()
{
    delete coloring;
    delete l_filter;
}


void
SubTraceWidget::compactToggle( bool _compact)
{
    if (_compact)
    {
	setMaximumHeight(25);     
        setMinimumHeight(25);
        setContentsMargins (0, 0, 0,0 );
    }else
    {
	setMaximumHeight(43);     
        setMinimumHeight(43);
        setContentsMargins (0, 0, 0,0 );
    }
}


void
SubTraceWidget::updateCursor(QPointF _p)
{
    if (isHidden())
        return;
    emit cursorMoved(_p);
    
//     uint64_t _s = z_filter->get_time_in_window( _p.x());
//     std::pair<double, uint16_t> _s_in_seconds = trace->convert_to_seconds( z_filter->get_zoom_start() );
//     std::pair<double, uint16_t> _f_s_in_seconds = trace->convert_to_seconds( z_filter->get_zoom_end() );
//     std::pair<double, uint16_t> _p_in_seconds = trace->convert_to_seconds( _s );
//     blade_service->updateValueWidget(   NULL,
//                                         _s_in_seconds.first  ,
//                                         _f_s_in_seconds.first ,
//                                         _p_in_seconds.first,
//                                         0.,
//                                         0.,
//                                         0.,
//                                         0.,
//                                         0.
//                                     );
// 
    
} 
    
    
QPointF
SubTraceWidget::getCursor()
{
   return view->getCursorPosition();
} 
          
         
void
SubTraceWidget::setCursor(QPointF& _p)
{
    if (isHidden())
        return;
    view->setCursorPosition( _p );
    if (metric_trace_widget != NULL)
        metric_trace_widget->setCursor(_p);
} 
              
    
    
void
SubTraceWidget::changeZoom()
{
    QPointF _p = view->getZoomF();
    z_filter->zoom_in( _p );

    resizeEvent(0);
}
  
void
SubTraceWidget::zoomingTip()
{
    if (isHidden())
        return;
    QPointF _p = view->getZoomF();
    uint64_t _s = z_filter->get_time_in_window( _p.x());
    uint64_t _f = z_filter->get_time_in_window( _p.y());
    std::pair<double, uint16_t> _s_in_seconds = trace->convert_to_seconds( _s );
    std::pair<double, uint16_t> _f_s_in_seconds = trace->convert_to_seconds( _f - _s );
    QString _tip = (QString(" From %1%2s").arg(_s_in_seconds.first).arg( QString::fromStdString(bladeservices::convertToUnit (_s_in_seconds.second)) ) +
    QString("+%1%2s").arg(_f_s_in_seconds.first).arg( QString::fromStdString(bladeservices::convertToUnit (_f_s_in_seconds.second)), 0, 'f' )
   );
    view->showToolTip( _tip );
    
}
      
  
  
void 
SubTraceWidget::resizeEvent(QResizeEvent *event)
{
    updateScene();
}





void 
SubTraceWidget::updateScene()
{
    if (isHidden())
        return;
    r_filter->set_available_pixels( view->width() );
    scene->clear();
    trace_view.clear();
    view->centerOn(QPointF(view->width()/2., view->height()/2.) );
    
    scene->addRect(QRectF(0  ,0 , view->width() , view->height()));
    
    trace_view = trace->get_trace_view(location, z_filter, cf_filter, s_filter, r_filter,l_filter, role_filter, rp_filter);    
    
    for( size_t i=0; i<trace_view.size() ; ++i)
    {
        TraceEvent * _te = trace_view[i];
        
        std::pair<QPen, QBrush> _colors = coloring->getColoring( _te) ;

        double _normalized_start =  (_te->get_start_timestamp() > z_filter->get_zoom_start())?
                                    ((double)(_te->get_start_timestamp() - z_filter->get_zoom_start())
                                    /
                                    (double)( z_filter->get_zoom_size() )):0;
                                    
        double _normalized_end =    (_te->get_end_timestamp()  < z_filter->get_zoom_size()+ z_filter->get_zoom_start())?
                                    ((double)(_te->get_end_timestamp() -  z_filter->get_zoom_start())
                                    /
                                    (double)(z_filter->get_zoom_size() )):1.;

        std::pair<uint64_t, uint64_t> _point=std::make_pair( (uint64_t)(_normalized_start * view->width()), (uint64_t)(_normalized_end * view->width()));

        if ( _te->is_exclusive())
        {
            QPen pen;
            pen.setColor(Qt::white);
            QBrush brush;
            brush.setColor( Qt::white );
            brush.setStyle( Qt::SolidPattern );
            scene->addRect( QRect( _point.first ,0, _point.second - _point.first , view->height()),pen, brush );
        }
        else
        {
            TraceEventRect * _rect = new TraceEventRect( trace, _te, QRect( _point.first ,0, _point.second - _point.first , view->height()) );
            _rect->setPen( _colors.first );
            _rect->setBrush( _colors.second );
            scene->addItem( _rect );
        }
        

    }
    
    view->updateScene();
    
    if (metric_trace_widget != NULL)
        metric_trace_widget->set_trace_view( trace_view );
    
}




