/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include "TraceEventGroupLateSenderAnalyzer.h"
#include "Service.h"
#include "Trace.h"

extern Trace* trace;

TraceEventGroupLateSenderAnalyzer::TraceEventGroupLateSenderAnalyzer(): TraceEventGroupAnalyzer()
{

    name = "Late Sender";
    description = "If the receiving process is\n"
                  "waiting for multiple messages\n"
                  "to arrive (e.g., in an call to\n"
                  "MPI_Waitall), the maximum waiting\n"
                  "time is accounted, i.e., the\n"
                  "waiting time due to the latest sender. ";
    
}

TraceEventGroupLateSenderAnalyzer::~TraceEventGroupLateSenderAnalyzer()
{
}


bool
TraceEventGroupLateSenderAnalyzer::probe(std::vector<TraceEvent*>& events)
{
    bool only_p2p = true;
    if (events.size() != 2) // only two communication partners
        return false;
    for (std::vector<TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        only_p2p &= (_e->get_region_role( ) == OTF2_REGION_ROLE_POINT2POINT  );
    }
    if (!only_p2p)
        return false;
    
    sender = dynamic_cast<SendP2PTraceEvent*>(events[0]);
    receiver = dynamic_cast<RecvP2PTraceEvent*>(events[1]);
    first_sit = true;
    if (sender == NULL || receiver == NULL) // first is not sender or second is not reciever
    {
        // try opposite
            sender = dynamic_cast<SendP2PTraceEvent*>(events[1]);
            receiver = dynamic_cast<RecvP2PTraceEvent*>(events[0]);
            if (sender == NULL || receiver == NULL) // first is not sender or second is not reciever
            {
                return false; // second option also doesn't work
            }
            first_sit = false;
    }
    if (sender->get_start_timestamp() < receiver->get_start_timestamp())
        return false;
    
    s_start = sender->get_start_timestamp();
    r_start = receiver->get_start_timestamp();
    
    return true;
}

void
TraceEventGroupLateSenderAnalyzer::analyze(std::vector<TraceEvent*>& events, int dh, int boxwidth )
{
    TraceEventGroupAnalyzer::analyze( events, dh, boxwidth  );
    clear_analyzer();
    
    int line_index = (first_sit)? 1:0;
    
    
     std::pair<double,double> _dp;
    _dp.first =  ((double)(r_start - group_min))/(double)(group_max - group_min);
    _dp.second = ((double)(s_start - group_min))/(double)(group_max - group_min);
    
    TraceEventRect * _rect = new TraceEventRect(s_start-r_start, trace, receiver , QRect(_dp.first * boxwidth, location_line_index[line_index]*dh, ( _dp.second - _dp.first)*boxwidth, dh) );

    QGraphicsSimpleTextItem * _dur_label = new QGraphicsSimpleTextItem( _rect->getDuration() );
    _dur_label->setPos(boxwidth+10,  location_line_index[line_index]*dh);
    dynamic_labels.push_back(_dur_label); 
    
    
    
    _rect->setPen( QColor(127, 0, 0, 200) );
    QBrush brush;
    brush.setColor( QColor(127, 0, 0, 200)  );
    brush.setStyle( Qt::SolidPattern );
    _rect->setBrush(brush );
            
    rects.push_back( _rect );
    

}


