/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <utility>
#include <stack>

#include <QLabel>
#include <QHBoxLayout>
#include <QTextEdit>
#include <QTabWidget>

#include "TraceInfoWidget.h"
#include "Service.h"


TraceInfoWidget::TraceInfoWidget(Trace * _trace, QWidget * _parent):QWidget(_parent), trace(_trace)
{
        QVBoxLayout* layout = new QVBoxLayout();       
        this->setLayout( layout );
        
        QTabWidget * _tab_widget = new QTabWidget();
        layout->addWidget(_tab_widget );
       
        general_info =  new QTextEdit();
        general_info->setReadOnly(true);
        _tab_widget->addTab( general_info, "General");
        
        general_info->setText(QString("Path : %1\n"
                                      "#Events : %2")
                            .arg( QString::fromStdString("Path : " + (trace->get_trace_path())))
                            .arg(trace->get_num_of_events())
                      );
        
        locations_info =  new QTextEdit();
        locations_info->setReadOnly(true);
        _tab_widget->addTab( locations_info, "System Tree");
         QString _system_tree_text = QString::fromStdString( trace->get_system_tree_description());
        locations_info->setText(_system_tree_text );
        
        std::pair<double, uint16_t> tl =trace->convert_to_seconds( trace->get_end_timestamp() );
        clock_info =  new QTextEdit(); 
        clock_info->setReadOnly(true);       
        _tab_widget->addTab( clock_info, "Clock");
        QString _text = QString("Start timestamp  : %1\n"
                                "End timestamp    : %2 (%3%4s) \n"
                                "Clock resolution : %5 \n"
        )
        .arg(trace->get_start_timestamp())
        .arg(trace->get_end_timestamp())
        .arg(tl.first)
        .arg( QString::fromStdString(bladeservices::convertToUnit(tl.second)))
        .arg(trace->get_timer_resolution());
        clock_info->setText(_text );
        
        
        
        regions_info =  new QTextEdit();
        regions_info->setReadOnly(true);     
        _tab_widget->addTab( regions_info, "Regions");
        QString _regions_text;
        for (size_t i=0; i<trace->get_num_of_regions(); ++i)
        {
            const RegionClass & _r = trace->get_region(i);
            _regions_text += QString::fromStdString (  _r.name ) + "\n";
        }
        regions_info->setText( _regions_text );
        
        calltree_info =  new QTextEdit();
        calltree_info->setReadOnly(true);     
        _tab_widget->addTab(calltree_info, "Call Tree");
        QString _calltree_text;
        const std::vector<CallPathClass>& calltree = trace->get_calltree();
        if (calltree.size()>0)
        {
            size_t level = 0;
            std::stack<int> _n_children;
            _n_children.push(-1);
            for (size_t i=0; i<calltree.size(); ++i)
            {
                const CallPathClass & _c = calltree[i];

                for (size_t indent=0; indent<level; ++indent)
                {
                    _calltree_text+="       ";
                }
                _calltree_text += QString("%1")
                                    .arg( QString::fromStdString (trace->get_region_name(_c.region)) )
                                    + "\n";
                if (_c.subcalltree.size()>0)
                {
                    _n_children.push(_c.subcalltree.size()-1);
                    level++;
                }
                else
                {
                    while(_n_children.size()>0 && _n_children.top()== 0)
                    {
                        _n_children.pop();
                        level--;
                    }
                    _n_children.top()--;
                }
            }
            calltree_info->setText(_calltree_text);
        }
        else
        {
            calltree_info->setText("Trace doesn't contain any call tree. Selections via metric tree and call tree are not working.");
            _tab_widget->setTabIcon( 4, QIcon(":/bladeicons/alert.png"));
            _tab_widget->setCurrentIndex(4);
        }
 
        if (trace->get_num_of_metric_members() >0)
        {
            metrics_info =  new QTextEdit();
            metrics_info->setReadOnly(true);     
            _tab_widget->addTab( metrics_info, "Metrics");
            QString _metrics_text;
            for (size_t i=0; i<trace->get_num_of_metric_members(); ++i)
            {
                const MetricMemberClass & _m = trace->get_metric_member(i);
                _metrics_text += QString::fromStdString (  _m.name ) + "\n";
            }
            metrics_info->setText( _metrics_text );
        }
 
 
            load_filters =  new LoadFiltersWidget(trace);
            
            connect(load_filters, SIGNAL(loadTrace()),
             this, SLOT(startLoadingTrace())); 
            
            _tab_widget->addTab( load_filters, "Load Trace");
            _tab_widget->setTabIcon( 6, QIcon(":/bladeicons/load_trace_icon.png"));

            
}
    
    
 
void
TraceInfoWidget::startLoadingTrace()   
{
    emit loadTrace();
}

 
TraceLoadEnterFilter *
TraceInfoWidget::get_load_enter_filter()
{
    return load_filters->get_trace_load_enter_filter();
}

 
TraceLoadLeaveFilter *
TraceInfoWidget::get_load_leave_filter()
{
    return load_filters->get_trace_load_leave_filter();
}

bool
TraceInfoWidget::do_grouping() const
{
    return load_filters->do_grouping();
}

