/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include "ZoomFilter.h"
#include "ResolutionFilter.h"


void
ZoomFilter::zoom_in( const QPointF& _p )
{
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    
    uint64_t _w = end_window - start_window;
    
    uint64_t _start_window = _p.x() * _w + start_window;
    uint64_t _end_window = _p.y() * _w + start_window;
    
    set_zoom_window( _start_window, _end_window);
}

void
ZoomFilter::unzoom(  )
{
    if (zoom.size()<=1)
        return;
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    
    uint64_t _w = end_window - start_window;    
    uint64_t _step = _w / 10;

    uint64_t _s = (start_window < _step)? 0: start_window - _step;
    uint64_t _f = (end_window + _step > zoom[0].second )? zoom[0].second: end_window + _step;
    zoom.back().first =_s;
    zoom.back().second =_f;

    refresh();
    emit filterChanged(); 
    emit filterChanged(get_window());
}



void
ZoomFilter::no_zoom(  )
{
    emit filterChanged(); 
    emit filterChanged(get_window());
}

void
ZoomFilter::go_back(  )
{
    if (zoom.size()<=1)
        return;
    zoom.pop_back(); 
    refresh();
    emit filterChanged(); 
    emit filterChanged(get_window());
}



void
ZoomFilter::move_left(  )
{
    if (zoom.size()<=1)
        return;
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    
    uint64_t _w = end_window - start_window;    
    uint64_t _step = _w / 10;

    uint64_t _s = (start_window < _step)? 0: start_window - _step;
    uint64_t _f = (start_window < _step)? _w: end_window - _step;
    zoom.back().first =_s;
    zoom.back().second =_f;

    refresh();
    emit filterChanged(); 
    emit filterChanged(get_window());
}

void
ZoomFilter::move_right(  )
{
    if (zoom.size()<=1)
        return;
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    
    uint64_t _w = end_window - start_window;    
    uint64_t _step = _w / 10;

    uint64_t _f = (end_window + _step > zoom[0].second)?  zoom[0].second : end_window + _step;
    uint64_t _s = (end_window + _step > zoom[0].second)?  zoom[0].second - _w : start_window + _step;
        
    zoom.back().first=_s;
    zoom.back().second=_f;
    refresh();
    emit filterChanged(); 
    emit filterChanged(get_window());
}

void
ZoomFilter::set_zoom_window( uint64_t _s, uint64_t _f)
{
    std::pair< uint64_t, uint64_t> _p = std::make_pair( _s, _f);
    zoom.push_back(_p );
    refresh();
    emit filterChanged();
    emit filterChanged(get_window());
}

Point
ZoomFilter::get_window( QPointF& _p)
{
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    
    uint64_t _w = end_window - start_window;
    
    Point p;
    p.first = ((uint64_t)( _w * _p.x()));
    p.second = ((uint64_t)( _w * _p.y()));
    return p;
    
}
 
QPointF
ZoomFilter::get_window( )
{
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    
    uint64_t _w = zoom[0].second -  zoom[0].first;
    
    QPointF p;
    p.setX( (double)start_window /(double) _w );
    p.setY( (double)end_window /(double) _w );
    
    return p;
    
} 
 
 
 
FilterResult 
ZoomFilter::apply( TraceEvent * _te )
{
    if (!is_active())
        return FILTER_OK;
    
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    if (_te->get_end_timestamp() < start_window )
        return FILTER_FAIL;
    if (_te->get_start_timestamp() > end_window )
        return FILTER_FAIL;
    
    return FILTER_OK;
    
}

uint64_t
ZoomFilter::get_time_in_window( double _f)
{
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
    uint64_t _w = end_window - start_window;
    
    return start_window + uint64_t(_f*_w);
    
}

void
ZoomFilter::refresh()
{
    uint64_t start_window = zoom.back().first;
    uint64_t end_window = zoom.back().second; 
 
    for (size_t i=0; i< get_num_children(); ++i)
    {
        ((ResolutionFilter* )(get_child(i)))->set_available_timestamps( end_window - start_window );
    }

}
