/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <algorithm> 

#include "TraceEvent.h"
#include "Trace.h"

extern Trace * trace;


TraceEvent::TraceEvent( ) // empty.. uses as dummy 
{
    location = (uint64_t)-1;
    region = (uint64_t)-1;
    
    caller = NULL;
    start_timestamp = (uint64_t)-1;
    end_timestamp = (uint64_t)-1;    
    visible = false;
    callpath = (uint32_t)-1;
    paradigm = OTF2_PARADIGM_UNKNOWN;
    exclusive = false;
    level = -1;
    events_group = NULL;
    suppressed = true;
    leader = false;
}


TraceEvent::TraceEvent( uint64_t _location)
{
    location = _location;
    region = (uint64_t)-1;
   
    caller = NULL;
    start_timestamp = (uint64_t)-1;
    end_timestamp = (uint64_t)-1;    
    visible = false;
    callpath = (uint32_t)-1;
    paradigm = OTF2_PARADIGM_UNKNOWN;
    exclusive = false;
    level = -1;
    events_group = NULL;
    suppressed = true;
    leader = false;
}

TraceEvent::~TraceEvent()
{
    for (std::vector<TraceEvent*>::iterator iter = sub_calls.begin(); iter != sub_calls.end(); ++iter)
    {
        delete *iter;
    }
}


std::string
TraceEvent::get_region_name() const
{
    return trace->get_region_name( get_region() ) ;
}

    
    
void
TraceEvent::set_level( int32_t _l)
{
    level = _l;
}

void
TraceEvent::set_visible( bool _v)
{
    visible = _v;
} 

void
TraceEvent::set_suppressed( bool _s)
{
    suppressed = _s;
} 

void
TraceEvent::set_group_leader( bool _s)
{
    leader = _s;
} 


void
TraceEvent::set_start( uint64_t _start)
{
    start_timestamp = _start;
}

void
TraceEvent::set_end( uint64_t _end)
{
    end_timestamp = _end;
}    
    
 
void
TraceEvent::set_start_metrics( std::vector<uint64_t>& _values )
{
    metric_start_value = _values;
}

void
TraceEvent::set_end_metrics( std::vector<uint64_t>&  _values)
{
    metric_end_value = _values;
}
     
  
 const std::vector<uint64_t>&
TraceEvent::get_start_metrics( ) const 
{
    return  metric_start_value;
}

const std::vector<uint64_t>& 
TraceEvent::get_end_metrics( ) const 
{
    return metric_end_value;
}
     
 
 
 
void
TraceEvent::set_region( uint64_t _r)
{
    region = _r;
}    
      
void
TraceEvent::set_callpath( uint32_t _c)
{
    callpath = _c;
}    
      
void
TraceEvent::set_paradigm( OTF2_Paradigm  _p)
{
    paradigm = _p;
}

OTF2_Paradigm 
TraceEvent::get_paradigm() const
{
    return paradigm;
};

void
TraceEvent::set_region_role( OTF2_RegionRole  _p)
{
    role = _p;
}

OTF2_RegionRole 
TraceEvent::get_region_role() const
{
    return role;
};


void
TraceEvent::add_sub_trace_event( TraceEvent * _trace_event)
{
    std::vector<TraceEvent*>::iterator iter = std::find(sub_calls.begin(),sub_calls.end(),  _trace_event);
    if (iter ==  sub_calls.end())
        sub_calls.push_back( _trace_event );
}

void
TraceEvent::remove_child( TraceEvent * _trace_event)
{
    std::vector<TraceEvent*>::iterator iter = std::find(sub_calls.begin(),sub_calls.end(),  _trace_event);
    if (iter !=  sub_calls.end())
	sub_calls.erase( iter );
}


void
TraceEvent::print(int level)
{
    for (int i=0; i<level; i++)
        std::cout << "  ";
    std::cout << get_region_name() << " s " << start_timestamp << std::endl;
    for (std::vector<TraceEvent*>::iterator iter = sub_calls.begin(); iter != sub_calls.end(); ++iter)
    {
        (*iter)->print( level + 1 );
    }
    for (int i=0; i<level; i++)
        std::cout << "  ";
    std::cout << get_region_name() << " f " << end_timestamp << std::endl;
}
    
std::pair< TraceEventPoint, TraceEventPoint> 
TraceEvent::get_trace_event_points()
{
    TraceEventPoint s;
    s.init(this, start_timestamp, true, !exclusive);
    TraceEventPoint f;
    f.init(this, end_timestamp, false, !exclusive);
    
    s.set_metric_values( get_start_metrics());
    f.set_metric_values( get_end_metrics());
    
    std::pair< TraceEventPoint, TraceEventPoint> pr = std::make_pair( s, f);
    
    return pr;
    
}
       

void
TraceEvent::reparent_children()
{
    for (std::vector<TraceEvent*>::iterator iter = sub_calls.begin(); iter != sub_calls.end(); ++iter)
    {
        caller->add_sub_trace_event( * iter );
    }
    if (caller != NULL)
        caller->remove_child( this );
    sub_calls.clear();
}
    
    
    
    
uint64_t location_extractor(const TraceEvent * _l)
{
    return _l->get_location();
}


    
    
