/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "config.h"
#include <QFile>
#include <QTextStream>
#include <QStringList>
#include <QRegularExpression>
#include <iostream>

#include "ScorePionFiltersFactory.h"
#include "ExcludeFileFilterRulesSet.h"
#include "ExcludeRegionFilterRulesSet.h"
#include "IncludeFileFilterRulesSet.h"
#include "IncludeRegionFilterRulesSet.h"
#include "DefaultFilterRulesSet.h"


namespace scorepion_plugin
{
    namespace filters
    {

        ScorePionFiltersFactory::ScorePionFiltersFactory(  )
        {
            default_filter_sets.push_back( new DefaultRulesSet());
        }



        void
         ScorePionFiltersFactory::readFilterFile(QString& filterfile)
        {


            enum FilterFilePosition {   OUTSIDE=0,
                                        FILE_BLOCK=1,
                                        FILE_BLOCK_EXCL=10,
                                        FILE_BLOCK_INCL=11,
                                        REGION_BLOCK=2,
                                        REGION_BLOCK_EXCL=20,
                                        REGION_BLOCK_INCL=21
            };

            QFile _file( filterfile );
            FilterRulesSet * current_filter_set = NULL;

            if (_file.open(QIODevice::ReadOnly))
            {
                QTextStream in(&_file);
                QString content = in.readAll().trimmed();
                _file.close();

                QRegularExpression filter_non_comment_mask( "\\\\#");
                content.replace(filter_non_comment_mask, "PAVEL SAVIAKOU IS GREATEST 3.14152659"); // hide
                QRegularExpression filter_comment( "#[^\n]+");
                content.replace(filter_comment, ""); // hide
                QRegularExpression filter_mask("PAVEL SAVIAKOU IS GREATEST 3.14152659");
                content.replace(filter_mask, "#"); // show back




#if QT_VERSION <= QT_VERSION_CHECK( 5, 14, 0 )
#define SKIP QString::SkipEmptyParts
#else
#define SKIP Qt::SkipEmptyParts
#endif

                QStringList filterlines = content.split(QRegularExpression("\\s+"),   SKIP  );

                QStringList::const_iterator constIterator = filterlines.constBegin();
                FilterFilePosition position = OUTSIDE;
                while ( constIterator != filterlines.constEnd() )
                {
                    // here we are if we are outside of region blocks
                    const QString & token = (*constIterator);

//                     std::cout << (*constIterator).toLocal8Bit().constData() << " ";

                    if (token == "SCOREP_REGION_NAMES_END" || token == "SCOREP_FILE_NAMES_END")
                    {
                        filter_sets.push_back( current_filter_set );
                        current_filter_set = NULL;
                        ++constIterator;
                        position = OUTSIDE;
                        continue; // next token
                    }
                    if (token == "SCOREP_REGION_NAMES_BEGIN" )
                    {
                        position = REGION_BLOCK;
                        ++constIterator;
                        continue; // next token
                    }

                    if (token == "SCOREP_FILE_NAMES_BEGIN" )
                    {
                        position = FILE_BLOCK;
                        ++constIterator;
                        continue; // next token
                    }
                    if (token == "MANGLED" )
                    {
                        ++constIterator;
                        continue; // next token
                    }
                    if (token == "INCLUDE" && !(position == FILE_BLOCK_INCL || position == REGION_BLOCK_INCL) )
                    {
                        if (position == FILE_BLOCK || position == FILE_BLOCK_EXCL )
                        {
                            position = FILE_BLOCK_INCL;

                            if (current_filter_set!= NULL)
                            {
                                filter_sets.push_back( current_filter_set );
                                current_filter_set = NULL;
                            }
                            current_filter_set= new IncludeFileFilterRulesSet();
                        }
                        else
                        {
                            position = REGION_BLOCK_INCL;
                            if (current_filter_set!= NULL)
                            {
                                filter_sets.push_back( current_filter_set );
                                current_filter_set = NULL;
                            }
                            current_filter_set= new IncludeRegionFilterRulesSet();
                        }
                        ++constIterator;
                        continue; // next token
                    }
                    if (token == "INCLUDE")
                    {
                        ++constIterator;
                        continue; // next token
                    }
                    if (token == "EXCLUDE" && !((position == FILE_BLOCK_EXCL || position == REGION_BLOCK_EXCL)))
                    {
                        if (position == FILE_BLOCK || position == FILE_BLOCK_INCL )
                        {
                            position = FILE_BLOCK_EXCL;

                            if (current_filter_set != NULL)
                            {
                                filter_sets.push_back( current_filter_set );
                                current_filter_set = NULL;
                            }
                            current_filter_set= new ExcludeFileFilterRulesSet();
                        }
                        else
                        {
                            position = REGION_BLOCK_EXCL;
                            if (current_filter_set != NULL)
                            {
                                filter_sets.push_back( current_filter_set );
                                current_filter_set = NULL;
                            }
                            current_filter_set= new ExcludeRegionFilterRulesSet();
                        }
                        ++constIterator;
                        continue; // next token
                    }
                    if (token == "EXCLUDE")
                    {
                        ++constIterator;
                        continue; // next token
                    }

                    if (current_filter_set != NULL)
                        current_filter_set->AddRule( (*constIterator).toStdString());
//                     std::cout << position << std::endl;
                    ++constIterator;
                }


            }

//             std::cout << " Size of filter set " << filter_sets.size() << std::endl;
//             for (
//                 std::list<FilterRulesSet *>::iterator iter = filter_sets.begin();
//                 iter != filter_sets.end();
//                 ++iter
//                 )
//             {
//                 (*iter)->print(std::cout);
//             }

        }



        std::list<FilterRulesSet *>&
        ScorePionFiltersFactory::getFilterSets()
        {
            return filter_sets;
        }

        std::list<FilterRulesSet *>&
        ScorePionFiltersFactory::getDefaultFilterSets()
        {
		return default_filter_sets;
        }

    }
 };

